-- DROP FUNCTION protis_add_interval;
-- DROP FUNCTION protis_timestamp_diff_sec;
-- DROP FUNCTION protis_timestamp_diff_str;

DELIMITER $$
CREATE FUNCTION protis_add_interval(
    p_date      TIMESTAMP,
    p_interval  INT,
    p_units     VARCHAR(10)
) RETURNS timestamp
    NO SQL
BEGIN
    DECLARE result TIMESTAMP;

    CASE p_units
        WHEN 'seconds' THEN SET result = p_date + INTERVAL p_interval second;
        WHEN 'minutes' THEN SET result = p_date + INTERVAL p_interval minute;
        WHEN 'hours'   THEN SET result = p_date + INTERVAL p_interval hour;
        WHEN 'days'    THEN SET result = p_date + INTERVAL p_interval day;
        WHEN 'months'  THEN SET result = p_date + INTERVAL p_interval month;
        WHEN 'years'   THEN SET result = p_date + INTERVAL p_interval year;
        ELSE SET result = p_date;
    END CASE;

    RETURN result;
END $$

DELIMITER $$
CREATE FUNCTION protis_timestamp_diff_sec(
    p_date1     TIMESTAMP,
    p_date2     TIMESTAMP
) RETURNS INT
    NO SQL
BEGIN
    RETURN TIMESTAMPDIFF(second, p_date1, p_date2);
END $$

DELIMITER $$
CREATE FUNCTION protis_timestamp_diff_str(
    p_date1     TIMESTAMP,
    p_date2     TIMESTAMP
) RETURNS VARCHAR(32)
    NO SQL
BEGIN
    DECLARE t INT;
    DECLARE d INT;
    DECLARE h INT;
    DECLARE m INT;
    DECLARE s INT;
    DECLARE sign VARCHAR(1);
    DECLARE st VARCHAR(32);

    SET sign = '';

    SET t = TIMESTAMPDIFF(second, p_date1, p_date2);
    IF t < 0 THEN
        SET t = -t;
        SET sign = '-';
    END IF;

    SET d = t div 86400;
    SET t = MOD(t, 3600 * 24);
    SET h = t div 3600;
    SET t = MOD(t, 3600);
    SET m = t div 60;
    SET s = MOD(t, 60);

    SET st = CONCAT(LPAD(h, 2, '0'), ':', LPAD(m, 2, '0'), ':', LPAD(s, 2, '0'));
    IF d = 0 THEN
        RETURN CONCAT(sign, st);
    ELSEIF d = 1 THEN
        RETURN CONCAT(sign, '1 day ', st);
    END IF;

    RETURN CONCAT(sign, d, ' days ', st);
END $$

