CREATE OR REPLACE FUNCTION protis_add_interval(
    p_date      TIMESTAMP,
    p_interval  varchar,
    p_units     VARCHAR
) RETURN timestamp
AS
    result TIMESTAMP;
BEGIN
    CASE p_units
        WHEN 'seconds' THEN result := p_date + p_interval * INTERVAL '1' second;
        WHEN 'minutes' THEN result := p_date + p_interval * INTERVAL '1' minute;
        WHEN 'hours'   THEN result := p_date + p_interval * INTERVAL '1' hour;
        WHEN 'days'    THEN result := p_date + p_interval * INTERVAL '1' day;
        WHEN 'months'  THEN result := p_date + p_interval * INTERVAL '1' month;
        WHEN 'years'   THEN result := p_date + p_interval * INTERVAL '1' year;
    END CASE;

    RETURN result;
END;

/

SHOW ERRORS;

CREATE OR REPLACE FUNCTION protis_timestamp_diff_sec (
    DATE_1 IN DATE, 
    DATE_2 IN DATE
) RETURN NUMBER IS

    NDATE_1   NUMBER;
    NDATE_2   NUMBER;
    NSECOND_1 NUMBER(10,0);
    NSECOND_2 NUMBER(10,0);

BEGIN
    -- Get Julian date number from first date (DATE_1)
    NDATE_1 := TO_NUMBER(TO_CHAR(DATE_1, 'J'));

    -- Get Julian date number from second date (DATE_2)
    NDATE_2 := TO_NUMBER(TO_CHAR(DATE_2, 'J'));

    -- Get seconds since midnight from first date (DATE_1)
    NSECOND_1 := TO_NUMBER(TO_CHAR(DATE_1, 'SSSSS'));

    -- Get seconds since midnight from second date (DATE_2)
    NSECOND_2 := TO_NUMBER(TO_CHAR(DATE_2, 'SSSSS'));

    RETURN (((NDATE_2 - NDATE_1) * 86400)+(NSECOND_2 - NSECOND_1));
END;
/

SHOW ERRORS;

CREATE OR REPLACE FUNCTION protis_timestamp_diff_str (
    DATE_1 IN DATE, 
    DATE_2 IN DATE
) RETURN VARCHAR IS

    NDATE_1   NUMBER;
    NDATE_2   NUMBER;
    NDAYS     NUMBER;
    NSECONDS  NUMBER(10,0);
    NSECOND_1 NUMBER(10,0);
    NSECOND_2 NUMBER(10,0);
    ST        VARCHAR(32);
    sign      VARCHAR(1) DEFAULT '';

BEGIN
    NDATE_1 := TO_NUMBER(TO_CHAR(DATE_1, 'J'));
    NDATE_2 := TO_NUMBER(TO_CHAR(DATE_2, 'J'));

    -- Get seconds since midnight from first date (DATE_1)
    NSECOND_1 := TO_NUMBER(TO_CHAR(DATE_1, 'SSSSS'));

    -- Get seconds since midnight from second date (DATE_2)
    NSECOND_2 := TO_NUMBER(TO_CHAR(DATE_2, 'SSSSS'));

    NSECONDS := (NDATE_2 - NDATE_1) * 86400 + NSECOND_2 - NSECOND_1;
    IF NSECONDS < 0 THEN
        sign := '-';
        NSECONDS := -NSECONDS;
    END IF;

    NDAYS := FLOOR(NSECONDS / 86400);

    ST := TO_CHAR(TO_DATE(MOD(NSECONDS, 86400),'SSSSS'), 'HH24:MI:SS');

    IF NDAYS = 0 THEN
        RETURN sign || ST;
    ELSIF NDAYS = 1 THEN
        RETURN sign || '1 day ' || ST;
    END IF;
    RETURN sign || NDAYS || ' days ' || ST;
END;
/

SHOW ERRORS;

SELECT protis_timestamp_diff_str(CURRENT_TIMESTAMP,CURRENT_TIMESTAMP - INTERVAL '3' DAY) FROM dual;

